/*
 * ll_dmadrivenusartcore.h
 *
 *  Created on: Nov 4, 2019
 *      Author: abody
 */

#ifndef LL_USARTCORE_H_
#define LL_USARTCORE_H_
#include <platform/usart_ll.h>

#include <f4ll/dma_helper.h>

namespace f4ll {

class usart_core
{
public:
    static inline void HandleUsartIrq(usart_core *_this) { _this->UsartIsr(); }
    static inline void HandleRxDmaIrq(usart_core *_this) { _this->RxDmaIsr(); }
    static inline void HandleTxDmaIrq(usart_core *_this) { _this->TxDmaIsr(); }

    void SetupTransmit(void const *buffer, uint16_t length);
    void SetupReceive(void *buffer, uint16_t length);

protected:
    usart_core(USART_TypeDef *usart, DMA_TypeDef *dma, uint32_t streamRx, uint32_t streamTx);

    USART_TypeDef *m_usart;
    dma_helper m_rxDma;
    dma_helper m_txDma;

private:
    virtual void ReceiverIdle(void) = 0;
    virtual void TransmissionComplete(void) = 0;
    virtual void FramingError(void) = 0;
    virtual void Overrun(void) = 0;

    virtual void RxDmaTransferComplete(void) = 0;
    virtual void RxDmaHalfTransfer(void) = 0;
    virtual void RxDmaError(dma_helper::DmaErrorType reason) = 0;

    virtual void TxDmaTransferComplete(void) = 0;
    virtual void TxDmaHalfTransfer(void) = 0;
    virtual void TxDmaError(dma_helper::DmaErrorType reason) = 0;

    void UsartIsr();
    void RxDmaIsr();
    void TxDmaIsr();
};

} /* namespace f4ll */

#endif /* LL_USARTCORE_H_ */
